import time

class UnwiredRequest:
	"""
	A request object for the Unwiredlabs location API.
	This class does all encoding that is needed.
	"""

	#
	# Public API
	#

	def __init__(self, device_id=None, key=None, mcc=None, mnc=None, fetch_address=False):
		"""
		Initialize an empty request

		:param device_id: device ID for this request if on a per device plan
		:param key: API key, may be skipped if set in ``UnwiredConnection``
		:param mcc: Mobile country code of the GSM Network, optional, shortcut if you want to skip it on adding Celltowers
		:param mnc: Mobile network code of the GSM Network, optional, shortcut if you want to skip it on adding Celltowers
		:param fetch_address: Set to ``true`` to fetch an address string with the response
		"""

		self.device_id = device_id
		self.aps = []
		self.gsmTowers = []
		self.lteTowers = []
		self.cdmaTowers = []
		self.umtsTowers = []
		self.gpsCoordinate = None
		self.mcc = mcc
		self.mnc = mnc
		self.key = key
		self.fetch_address = fetch_address

	def __str__(self):
		return '<UnwiredRequest: {wifi} WIFI APs, {tower} cell towers{gps}>'.format(
			wifi=len(self.aps),
			tower=len(self.gsmTowers) + len(self.lteTowers) + len(self.cdmaTowers) + len(self.umtsTowers),
			gps=', GPS: {lat}, {lon}'.format(lat=self.gpsCoordinate['lat'], lon=self.gpsCoordinate['lon']) if self.gpsCoordinate else ''
		)

	def addAccessPoint(self, BSSID, rssi, band='unknown'):
		"""
		Add WIFI station to search request

		:param BSSID: WIFI BSSID, hex encoded
		:param rssi: signal strength in dBm
		:param band: WIFI band, one of 'unknown', '2.4', 5.0'
		"""

		self.aps.append({
			'bssid': BSSID,
			'rssi': rssi,
			'band': band
		})

	def addGSMCellTower(self, lac, cellID, rssi, mcc=None, mnc=None):
		"""
		Add GSM Cell tower to search request

		:param lac: location area code
		:param cellID: celltower ID
		:param rssi: signal strength in dBm
		:param mcc: mobile country code, may be skipped if set on request initialization
		:param mnc: mobile network code, may be skipped if set on request initialization
		"""

		if self.mcc is None and mcc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if self.mnc is None and mnc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if mcc is None:
			mcc = self.mcc
		if mnc is None:
			mnc = self.mnc

		self.gsmTowers.append({
			'mcc': mcc,
			'mnc': mnc,
			'lac': lac,
			'cellID': cellID,
			'rssi': rssi
		})

	def addLTECellTower(self, lac, cellID, rssi, mcc=None, mnc=None):
		"""
		Add LTE Cell tower to search request

		:param lac: location area code
		:param cellID: celltower ID
		:param rssi: signal strength in dBm
		:param mcc: mobile country code, may be skipped if set on request initialization
		:param mnc: mobile network code, may be skipped if set on request initialization
		"""

		if self.mcc is None and mcc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if self.mnc is None and mnc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if mcc is None:
			mcc = self.mcc
		if mnc is None:
			mnc = self.mnc

		self.lteTowers.append({
			'mcc': mcc,
			'mnc': mnc,
			'lac': lac,
			'cellID': cellID,
			'rssi': rssi
		})

	def addCDMACellTower(self, lac, cellID, rssi, mcc=None, mnc=None):
		"""
		Add CDMA Cell tower to search request

		:param lac: location area code
		:param cellID: celltower ID
		:param rssi: signal strength in dBm
		:param mcc: mobile country code, may be skipped if set on request initialization
		:param mnc: mobile network code, may be skipped if set on request initialization
		"""

		if self.mcc is None and mcc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if self.mnc is None and mnc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if mcc is None:
			mcc = self.mcc
		if mnc is None:
			mnc = self.mnc

		self.cdmaTowers.append({
			'mcc': mcc,
			'mnc': mnc,
			'lac': lac,
			'cellID': cellID,
			'rssi': rssi
		})

	def addUMTSCellTower(self, lac, cellID, rssi, mcc=None, mnc=None):
		"""
		Add UMTS Cell tower to search request

		:param lac: location area code
		:param cellID: celltower ID
		:param rssi: signal strength in dBm
		:param mcc: mobile country code, may be skipped if set on request initialization
		:param mnc: mobile network code, may be skipped if set on request initialization
		"""

		if self.mcc is None and mcc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if self.mnc is None and mnc is None:
			raise RuntimeError('Either set mcc on initialization or provide it when adding a cell tower')
		if mcc is None:
			mcc = self.mcc
		if mnc is None:
			mnc = self.mnc

		self.umtsTowers.append({
			'mcc': mcc,
			'mnc': mnc,
			'lac': lac,
			'cellID': cellID,
			'rssi': rssi
		})

	def setGPSCoordinate(self, lat, lon, numSatelites, altitude=None, speed=None, hpe=0):
		"""
		Add a GPS coordinate to help Skyhook make their DB better

		:param lat: latitude, positive for South
		:param lon: longitude, positive for East
		:param numSatelites: number of visible satelites
		:param altitude: altitude above null, optional
		:param speed: speed, optional
		:param hpe: precision in meters, optional
		"""

		self.gpsCoordinate = {
			'latitude': lat,
			'longitude': lon,
			'accuracy': hpe if hpe != 0 else 50
		}

		if speed is not None:
			self.gpsCoordinate['speed'] = speed
		if altitude is not None:
			self.gpsCoordinate['altitude'] = altitude


	def serialize(self, key=None):
		"""
		Serialize a request into it's json form

		:param key: API key, optional if set on initialization
		"""

		result = {}

		if key:
			self.key = key

		if not self.key:
			raise RuntimeError('No API key set')

		result['token'] = self.key
		if self.device_id:
			result['id'] = self.device_id

		# radio type
		if len(self.gsmTowers)  > 0 and len(self.umtsTowers) == 0 and len(self.lteTowers) == 0 and len(self.cdmaTowers) == 0:
			result['radio'] = 'gsm'
		if len(self.gsmTowers) == 0 and len(self.umtsTowers)  > 0 and len(self.lteTowers) == 0 and len(self.cdmaTowers) == 0:
			result['radio'] = 'umts'
		if len(self.gsmTowers) == 0 and len(self.umtsTowers) == 0 and len(self.lteTowers)  > 0 and len(self.cdmaTowers) == 0:
			result['radio'] = 'lte'
		if len(self.gsmTowers) == 0 and len(self.umtsTowers) == 0 and len(self.lteTowers) == 0 and len(self.cdmaTowers)  > 0:
			result['radio'] = 'cdma'

		result['cells'] = self.serializeCellTower()
		result['address'] = 1 if self.fetch_address else 0
		result['wifi'] = self.serializeAP()
		result['fallbacks'] = ['lacf', 'scf']
		# result['ip'] = self.ip
		result['bt'] = 0  # only land coordinates

		return result

	def serialize_contrib(self):
		"""
		Serialize a request into it's json form for the contribute API

		:param key: API key, optional if set on initialization
		"""

		if not self.gpsCoordinate:
			return None

		result = {}

		result['timestamp'] = int(time.time())
		result['position'] = self.gpsCoordinate
		result['cells'] = self.serializeCellTower()
		result['wifi'] = self.serializeAP()

		return {"items": [ result ] }

	#
	# Internal
	#

	def serializeAP(self):
		"""
		Serialize WIFI APs for payload

		:return: Serialized WIFI APs as array
		"""

		result = []
		for ap in self.aps:
			item = {
				"bssid": ap['bssid'],
				"signal": ap['rssi']
			}
			result.append(item)

		return result


	def serializeCellTower(self):
		"""
		Serialize cell tower data for payload

		:return: Serialized cell tower payload as array
		"""

		result = []
		towers_list = [
			('gsm', self.gsmTowers),
			('lte', self.lteTowers),
			('cdma', self.cdmaTowers),
			('umts', self.umtsTowers)
		]
		for radio, towers in towers_list:
			for tower in towers:
				item = {
					"radio": radio,
					"mcc": tower['mcc'],
					"mnc": tower['mnc'],
					"lac": tower['lac'],
					"cid": tower['cellID'],

				}
				if tower['rssi'] <= -51 and tower['rssi'] >= -113:
					item['signal'] = tower['rssi']
				result.append(item)
		return result



import requests
from .response import UnwiredResponse


class UnwiredConnection:
	"""
	Unwiredlabs server connection class.

	You may specify a custom server by supplying ``server``, defaults to the EU server.
	If you only use one account on that connection you may specify your API Key ``key``
	while initializing this
	"""

	def __init__(self, server='eu1.unwiredlabs.com', key=None, contribute_key=None, contribute_sandbox=True):
		self.session = requests.Session()
		self.server = server
		self.key = key
		self.contribute_key = contribute_key
		self.contribute_sandbox = contribute_sandbox

	def performRequest(self, request):
		"""
		Perform a request on the connection and return the response, blocks until response received
		This may throw network and runtime errors when something goes wrong.

		:param request: the request to perform
		:return: ``UnwiredResponse`` object with the response
		"""
		headers = {
			"ContentType": "application/json",
			"Accept": "application/json"
		}
		request = requests.Request(
			"POST",
			"https://{}/v2/process.php".format(self.server),
			headers=headers,
			json=request.serialize(key=self.key)
		)
		prep = self.session.prepare_request(request)
		response = self.session.send(prep)
		return UnwiredResponse(response)

	def performContributeRequest(self, request):
		"""
		Perform a contribution API request, only works for requests that contain a GPS coordinate
		and a device ID. Setup ``contribute_key`` before calling.
		Will return 400 on requests that do not match the requirements.

		:param request: the request to perform
		:return: HTTP Status code of the response
		"""
		if not request.gpsCoordinate or not request.device_id or not self.contribute_key:
			return 400 # Do not send back data without GPS

		headers = {
			"ContentType": "application/json",
			"Accept": "application/json"
		}
		request = requests.Request(
			"POST",
			"https://d1.unwiredlabs.com/geosubmit.php?sbox={sandbox}&key={key}&id={device}".format(
				sandbox=1 if self.contribute_sandbox else 0,
				key=self.contribute_key,
				device=request.device_id
			),
			headers=headers,
			json=request.serialize_contrib()
		)
		prep = self.session.prepare_request(request)
		response = self.session.send(prep)
		return response.status_code




# -*- coding: utf-8 -*-

import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

exec(open('unwiredlabs/version.py').read())

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='unwiredlabs',
    version=__version__,
    packages=['unwiredlabs'],
    include_package_data=True,
    license='BSD License',  # example license
    description='Unwiredlabs Location API client.',
    long_description=README,
    url='https://github.com/anfema/python-unwired',
    author='Johannes Schriewer',
    author_email='j.schriewer@anfe.ma',
    install_requires=[
        "requests >= 2.10.0",
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
    ],
)



import datetime


class InvalidDataError(RuntimeError):
	"""
	Error class raised when the payload received from the Unwiredlabs server is somehow corrupt
	"""

	pass

class UnwiredResponse:
	"""
	Response deserializer class
	"""

	#
	# Public API
	#

	def __init__(self, data):
		"""
		Initialize the response class with data

		:param data: the data from the Unwiredlabs response
		"""

		if not data:
			raise RuntimeError('No response data set')

		self.data = data.json()
		self.status = 'Undefined'
		self.lat = None
		self.lon = None
		self.hpe = None
		self.date = None
		self.balance = None
		self.address = None
		self.deserialize()

	@property
	def coordinate(self):
		"""
		Fetch coordinate

		:return: Coordinate tuple (lat,long) if payload was not an error, else ``None``
		"""
		if self.status == 'Ok':
			return (self.lat, self.lon)
		else:
			return None

	#
	# Internal
	#

	def deserialize(self):
		"""
		Deserialize a response
		"""

		self.status = self.data['status'].title()
		if self.status == 'Ok':
			self.lat = self.data['lat']
			self.lon = self.data['lon']
			self.hpe = self.data['accuracy']
			self.balance = self.data['balance']
			self.address = self.data.get('address', None)
		else:
			self.status = self.data['message']
